@@ -15,6 +15,7 @@ import static com.google.inject.matcher.Matchers.subclassesOf;
 import static org.eclipse.che.inject.Matchers.names;
 import static org.eclipse.che.multiuser.api.permission.server.SystemDomain.SYSTEM_DOMAIN_ACTIONS;
 
+import com.auth0.jwk.JwkProvider;
 import com.google.inject.AbstractModule;
 import com.google.inject.TypeLiteral;
 import com.google.inject.assistedinject.FactoryModuleBuilder;
@@ -22,7 +23,7 @@ import com.google.inject.multibindings.MapBinder;
 import com.google.inject.multibindings.Multibinder;
 import com.google.inject.name.Names;
 import io.jsonwebtoken.JwtParser;
-import io.jsonwebtoken.impl.DefaultJwtParser;
+import io.jsonwebtoken.SigningKeyResolver;
 import java.util.HashMap;
 import java.util.Map;
 import javax.sql.DataSource;
@@ -81,7 +82,6 @@ import org.eclipse.che.commons.observability.deploy.ExecutorWrapperModule;
 import org.eclipse.che.core.db.DBTermination;
 import org.eclipse.che.core.db.schema.SchemaInitializer;
 import org.eclipse.che.core.tracing.metrics.TracingMetricsModule;
-import org.eclipse.che.inject.ConfigurationException;
 import org.eclipse.che.inject.DynaModule;
 import org.eclipse.che.multiuser.api.authentication.commons.token.ChainedTokenExtractor;
 import org.eclipse.che.multiuser.api.authentication.commons.token.HeaderRequestTokenExtractor;
@@ -93,6 +93,11 @@ import org.eclipse.che.multiuser.api.workspace.activity.MultiUserWorkspaceActivi
 import org.eclipse.che.multiuser.keycloak.server.deploy.KeycloakModule;
 import org.eclipse.che.multiuser.keycloak.server.deploy.KeycloakUserRemoverModule;
 import org.eclipse.che.multiuser.machine.authentication.server.MachineAuthModule;
+import org.eclipse.che.multiuser.oidc.OIDCInfo;
+import org.eclipse.che.multiuser.oidc.OIDCInfoProvider;
+import org.eclipse.che.multiuser.oidc.OIDCJwkProvider;
+import org.eclipse.che.multiuser.oidc.OIDCJwtParserProvider;
+import org.eclipse.che.multiuser.oidc.OIDCSigningKeyResolver;
 import org.eclipse.che.multiuser.organization.api.OrganizationApiModule;
 import org.eclipse.che.multiuser.organization.api.OrganizationJpaModule;
 import org.eclipse.che.multiuser.permission.user.UserServicePermissionsFilter;
@@ -335,18 +340,10 @@ public class WsMasterModule extends AbstractModule {
           .to(org.eclipse.che.api.workspace.server.DefaultWorkspaceStatusCache.class);
     }
 
-    if (OpenShiftInfrastructure.NAME.equals(infrastructure)) {
-      if (Boolean.parseBoolean(System.getenv("CHE_AUTH_NATIVEUSER"))) {
-        bind(KubernetesClientConfigFactory.class).to(KubernetesOidcProviderConfigFactory.class);
-      } else {
-        bind(KubernetesClientConfigFactory.class).to(KeycloakProviderConfigFactory.class);
-      }
-    }
-
-    if (KubernetesInfrastructure.NAME.equals(infrastructure)
-        && Boolean.parseBoolean(System.getenv("CHE_AUTH_NATIVEUSER"))) {
-      throw new ConfigurationException(
-          "Native user mode is not supported on Kubernetes. It is supported only on OpenShift.");
+    if (Boolean.parseBoolean(System.getenv("CHE_AUTH_NATIVEUSER"))) {
+      bind(KubernetesClientConfigFactory.class).to(KubernetesOidcProviderConfigFactory.class);
+    } else if (OpenShiftInfrastructure.NAME.equals(infrastructure)) {
+      bind(KubernetesClientConfigFactory.class).to(KeycloakProviderConfigFactory.class);
     }
 
     persistenceProperties.put(
@@ -395,11 +392,16 @@ public class WsMasterModule extends AbstractModule {
     install(new OrganizationJpaModule());
 
     if (Boolean.parseBoolean(System.getenv("CHE_AUTH_NATIVEUSER"))) {
+      bind(RequestTokenExtractor.class).to(HeaderRequestTokenExtractor.class);
+      if (KubernetesInfrastructure.NAME.equals(infrastructure)) {
+        bind(OIDCInfo.class).toProvider(OIDCInfoProvider.class).asEagerSingleton();
+        bind(SigningKeyResolver.class).to(OIDCSigningKeyResolver.class);
+        bind(JwtParser.class).toProvider(OIDCJwtParserProvider.class);
+        bind(JwkProvider.class).toProvider(OIDCJwkProvider.class);
+      }
       bind(TokenValidator.class).to(NotImplementedTokenValidator.class);
-      bind(JwtParser.class).to(DefaultJwtParser.class);
       bind(ProfileDao.class).to(JpaProfileDao.class);
       bind(OAuthAPI.class).to(EmbeddedOAuthAPI.class);
-      bind(RequestTokenExtractor.class).to(HeaderRequestTokenExtractor.class);
     } else {
       install(new KeycloakModule());
       install(new KeycloakUserRemoverModule());
