@@ -19,6 +19,8 @@ import org.eclipse.che.inject.ConfigurationException;
 import org.eclipse.che.inject.DynaModule;
 import org.eclipse.che.multiuser.keycloak.server.deploy.KeycloakServletModule;
 import org.eclipse.che.multiuser.machine.authentication.server.MachineLoginFilter;
+import org.eclipse.che.multiuser.oidc.filter.OidcTokenInitializationFilter;
+import org.eclipse.che.workspace.infrastructure.kubernetes.KubernetesInfrastructure;
 import org.eclipse.che.workspace.infrastructure.openshift.OpenShiftInfrastructure;
 import org.eclipse.che.workspace.infrastructure.openshift.multiuser.oauth.OpenshiftTokenInitializationFilter;
 import org.everrest.guice.servlet.GuiceEverrestServlet;
@@ -78,6 +80,8 @@ public class WsMasterServletModule extends ServletModule {
     final String infrastructure = System.getenv("CHE_INFRASTRUCTURE_ACTIVE");
     if (OpenShiftInfrastructure.NAME.equals(infrastructure)) {
       filter("/*").through(OpenshiftTokenInitializationFilter.class);
+    } else if (KubernetesInfrastructure.NAME.equals(infrastructure)) {
+      filter("/*").through(OidcTokenInitializationFilter.class);
     } else {
       throw new ConfigurationException("Native user mode is currently supported on on OpenShift.");
     }
