@@ -14,10 +14,12 @@ package org.eclipse.che.workspace.infrastructure.kubernetes.namespace;
 import static java.util.Collections.emptyMap;
 import static java.util.Collections.emptySet;
 import static java.util.Collections.singletonList;
+import static java.util.Optional.empty;
 import static org.eclipse.che.api.workspace.shared.Constants.WORKSPACE_INFRASTRUCTURE_NAMESPACE_ATTRIBUTE;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta.DEFAULT_ATTRIBUTE;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta.PHASE_ATTRIBUTE;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.AbstractWorkspaceServiceAccount.CREDENTIALS_SECRET_NAME;
+import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.AbstractWorkspaceServiceAccount.PREFERENCES_CONFIGMAP_NAME;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.AbstractWorkspaceServiceAccount.SECRETS_ROLE_NAME;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.KubernetesNamespaceFactory.NAMESPACE_TEMPLATE_ATTRIBUTE;
 import static org.mockito.ArgumentMatchers.any;
@@ -41,6 +43,7 @@ import static org.testng.Assert.fail;
 import ch.qos.logback.classic.spi.LoggingEvent;
 import ch.qos.logback.core.Appender;
 import com.google.common.collect.ImmutableMap;
+import io.fabric8.kubernetes.api.model.ConfigMap;
 import io.fabric8.kubernetes.api.model.Namespace;
 import io.fabric8.kubernetes.api.model.NamespaceBuilder;
 import io.fabric8.kubernetes.api.model.NamespaceList;
@@ -87,6 +90,10 @@ import org.eclipse.che.workspace.infrastructure.kubernetes.CheServerKubernetesCl
 import org.eclipse.che.workspace.infrastructure.kubernetes.KubernetesClientFactory;
 import org.eclipse.che.workspace.infrastructure.kubernetes.api.server.impls.KubernetesNamespaceMetaImpl;
 import org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta;
+import org.eclipse.che.workspace.infrastructure.kubernetes.namespace.configurator.CredentialsSecretConfigurator;
+import org.eclipse.che.workspace.infrastructure.kubernetes.namespace.configurator.NamespaceConfigurator;
+import org.eclipse.che.workspace.infrastructure.kubernetes.namespace.configurator.PreferencesConfigMapConfigurator;
+import org.eclipse.che.workspace.infrastructure.kubernetes.namespace.configurator.WorkspaceServiceAccountConfigurator;
 import org.eclipse.che.workspace.infrastructure.kubernetes.provision.NamespaceProvisioner;
 import org.eclipse.che.workspace.infrastructure.kubernetes.util.KubernetesSharedPool;
 import org.mockito.ArgumentCaptor;
@@ -144,6 +151,7 @@ public class KubernetesNamespaceFactoryTest {
     serverMock.before();
     k8sClient = spy(serverMock.getClient());
     lenient().when(clientFactory.create()).thenReturn(k8sClient);
+    lenient().when(cheClientFactory.create()).thenReturn(k8sClient);
     lenient().when(k8sClient.namespaces()).thenReturn(namespaceOperation);
 
     lenient().when(namespaceOperation.withName(any())).thenReturn(namespaceResource);
@@ -170,14 +178,13 @@ public class KubernetesNamespaceFactoryTest {
       throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -198,14 +205,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -219,14 +225,13 @@ public class KubernetesNamespaceFactoryTest {
   public void shouldNormaliseNamespaceWhenUserNameStartsWithKube() {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -245,14 +250,13 @@ public class KubernetesNamespaceFactoryTest {
           throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -268,14 +272,13 @@ public class KubernetesNamespaceFactoryTest {
   public void shouldThrowExceptionIfNoDefaultNamespaceIsConfigured() {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             null,
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -319,14 +322,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -363,14 +365,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -394,14 +395,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -428,14 +428,13 @@ public class KubernetesNamespaceFactoryTest {
             .build());
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -457,14 +456,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -488,27 +486,27 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                Set.of(new CredentialsSecretConfigurator(clientFactory)),
                 clientFactory,
                 cheClientFactory,
                 userManager,
                 preferenceManager,
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
-    KubernetesSecrets secrets = mock(KubernetesSecrets.class);
-    when(toReturnNamespace.secrets()).thenReturn(secrets);
-    when(secrets.get()).thenReturn(Collections.emptyList());
+    when(toReturnNamespace.getName()).thenReturn("namespaceName");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
     MixedOperation mixedOperation = mock(MixedOperation.class);
-    lenient().when(k8sClient.secrets()).thenReturn(mixedOperation);
-    lenient().when(mixedOperation.inNamespace(anyString())).thenReturn(namespaceOperation);
+    when(k8sClient.secrets()).thenReturn(mixedOperation);
+    when(mixedOperation.inNamespace(anyString())).thenReturn(namespaceOperation);
+    when(namespaceResource.get()).thenReturn(null);
+    when(cheClientFactory.create()).thenReturn(k8sClient);
+    when(clientFactory.create()).thenReturn(k8sClient);
 
     // when
     RuntimeIdentity identity =
@@ -524,19 +522,98 @@ public class KubernetesNamespaceFactoryTest {
   }
 
   @Test
-  public void shouldNotCreateCredentialsSecretIfExists() throws Exception {
+  public void shouldCreatePreferencesConfigmapIfNotExists() throws Exception {
     // given
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                Set.of(new PreferencesConfigMapConfigurator(clientFactory)),
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    when(toReturnNamespace.getName()).thenReturn("namespaceName");
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    MixedOperation mixedOperation = mock(MixedOperation.class);
+    when(k8sClient.configMaps()).thenReturn(mixedOperation);
+    when(mixedOperation.inNamespace(anyString())).thenReturn(namespaceOperation);
+    when(namespaceResource.get()).thenReturn(null);
+
+    // when
+    RuntimeIdentity identity =
+        new RuntimeIdentityImpl("workspace123", null, USER_ID, "workspace123");
+    namespaceFactory.getOrCreate(identity);
+
+    // then
+    ArgumentCaptor<ConfigMap> configMapCaptor = ArgumentCaptor.forClass(ConfigMap.class);
+    verify(namespaceOperation).create(configMapCaptor.capture());
+    ConfigMap configmap = configMapCaptor.getValue();
+    Assert.assertEquals(configmap.getMetadata().getName(), PREFERENCES_CONFIGMAP_NAME);
+  }
+
+  @Test
+  public void testAllConfiguratorsAreCalledWhenCreatingNamespace() throws InfrastructureException {
+    // given
+    String namespaceName = "testNamespaceName";
+    NamespaceConfigurator configurator1 = Mockito.mock(NamespaceConfigurator.class);
+    NamespaceConfigurator configurator2 = Mockito.mock(NamespaceConfigurator.class);
+    Set<NamespaceConfigurator> namespaceConfigurators = Set.of(configurator1, configurator2);
+
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                namespaceConfigurators,
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    EnvironmentContext.getCurrent().setSubject(new SubjectImpl("jondoe", "123", null, false));
+
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    when(toReturnNamespace.getName()).thenReturn(namespaceName);
+
+    RuntimeIdentity identity = new RuntimeIdentityImpl("workspace123", null, USER_ID, "old-che");
+    doReturn(toReturnNamespace).when(namespaceFactory).get(identity);
+
+    // when
+    KubernetesNamespace namespace = namespaceFactory.getOrCreate(identity);
+
+    // then
+    NamespaceResolutionContext resolutionCtx =
+        new NamespaceResolutionContext("workspace123", "123", "jondoe");
+    verify(configurator1).configure(resolutionCtx, namespaceName);
+    verify(configurator2).configure(resolutionCtx, namespaceName);
+    assertEquals(namespace, toReturnNamespace);
+  }
+
+  @Test
+  public void shouldNotCreateCredentialsSecretIfExists() throws Exception {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -558,6 +635,40 @@ public class KubernetesNamespaceFactoryTest {
     verify(namespaceOperation, never()).create(any());
   }
 
+  @Test
+  public void shouldNotCreatePreferencesConfigmapIfExists() throws Exception {
+    // given
+    namespaceFactory =
+        spy(
+            new KubernetesNamespaceFactory(
+                "<username>-che",
+                true,
+                true,
+                true,
+                NAMESPACE_LABELS,
+                NAMESPACE_ANNOTATIONS,
+                emptySet(),
+                clientFactory,
+                cheClientFactory,
+                userManager,
+                preferenceManager,
+                pool));
+    KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
+    prepareNamespace(toReturnNamespace);
+    doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
+    MixedOperation mixedOperation = mock(MixedOperation.class);
+    lenient().when(k8sClient.configMaps()).thenReturn(mixedOperation);
+    lenient().when(mixedOperation.inNamespace(anyString())).thenReturn(namespaceOperation);
+
+    // when
+    RuntimeIdentity identity =
+        new RuntimeIdentityImpl("workspace123", null, USER_ID, "workspace123");
+    namespaceFactory.getOrCreate(identity);
+
+    // then
+    verify(namespaceOperation, never()).create(any());
+  }
+
   @Test(
       expectedExceptions = InfrastructureException.class,
       expectedExceptionsMessageRegExp =
@@ -565,14 +676,13 @@ public class KubernetesNamespaceFactoryTest {
   public void shouldThrowExceptionWhenFailedToGetInfoAboutDefaultNamespace() throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -591,14 +701,13 @@ public class KubernetesNamespaceFactoryTest {
   public void shouldThrowExceptionWhenFailedToGetNamespaces() throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -622,14 +731,13 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -645,7 +753,6 @@ public class KubernetesNamespaceFactoryTest {
 
     // then
     assertEquals(toReturnNamespace, namespace);
-    verify(namespaceFactory, never()).doCreateServiceAccount(any(), any());
     verify(toReturnNamespace).prepare(eq(false), any(), any());
   }
 
@@ -655,14 +762,13 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-che",
                 false,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -679,7 +785,6 @@ public class KubernetesNamespaceFactoryTest {
 
     // then
     assertEquals(toReturnNamespace, namespace);
-    verify(namespaceFactory, never()).doCreateServiceAccount(any(), any());
     verify(toReturnNamespace).prepare(eq(false), any(), any());
   }
 
@@ -687,17 +792,18 @@ public class KubernetesNamespaceFactoryTest {
   public void shouldPrepareWorkspaceServiceAccountIfItIsConfiguredAndNamespaceIsNotPredefined()
       throws Exception {
     // given
+    var serviceAccountCfg =
+        spy(new WorkspaceServiceAccountConfigurator("serviceAccount", "", clientFactory));
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "serviceAccount",
-                "",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                Set.of(serviceAccountCfg),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -705,13 +811,12 @@ public class KubernetesNamespaceFactoryTest {
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
     prepareNamespace(toReturnNamespace);
-    when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
 
     KubernetesWorkspaceServiceAccount serviceAccount =
         mock(KubernetesWorkspaceServiceAccount.class);
-    doReturn(serviceAccount).when(namespaceFactory).doCreateServiceAccount(any(), any());
+    doReturn(serviceAccount).when(serviceAccountCfg).doCreateServiceAccount(any(), any());
 
     // when
     RuntimeIdentity identity =
@@ -719,24 +824,25 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory.getOrCreate(identity);
 
     // then
-    verify(namespaceFactory).doCreateServiceAccount("workspace123", "workspace123");
+    verify(serviceAccountCfg).doCreateServiceAccount("workspace123", "workspace123");
     verify(serviceAccount).prepare();
   }
 
   @Test
   public void shouldBindToAllConfiguredClusterRoles() throws Exception {
     // given
+    var serviceAccountConfigurator =
+        new WorkspaceServiceAccountConfigurator("serviceAccount", "cr2, cr3", clientFactory);
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "serviceAccount",
-                "cr2, cr3",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                Set.of(serviceAccountConfigurator),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -744,10 +850,10 @@ public class KubernetesNamespaceFactoryTest {
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
     prepareNamespace(toReturnNamespace);
-    when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
     when(k8sClient.supportsApiPath(eq("/apis/metrics.k8s.io"))).thenReturn(true);
+    when(cheClientFactory.create()).thenReturn(k8sClient);
     when(clientFactory.create(any())).thenReturn(k8sClient);
 
     // pre-create the cluster roles
@@ -770,7 +876,6 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory.getOrCreate(identity);
 
     // then
-    verify(namespaceFactory).doCreateServiceAccount("workspace123", "workspace123");
 
     ServiceAccountList sas = k8sClient.serviceAccounts().inNamespace("workspace123").list();
     assertEquals(sas.getItems().size(), 1);
@@ -779,7 +884,12 @@ public class KubernetesNamespaceFactoryTest {
     RoleList roles = k8sClient.rbac().roles().inNamespace("workspace123").list();
     assertEquals(
         roles.getItems().stream().map(r -> r.getMetadata().getName()).collect(Collectors.toSet()),
-        Sets.newHashSet("workspace-view", "workspace-metrics", "workspace-secrets", "exec"));
+        Sets.newHashSet(
+            "workspace-configmaps",
+            "workspace-view",
+            "workspace-metrics",
+            "workspace-secrets",
+            "exec"));
     RoleBindingList bindings = k8sClient.rbac().roleBindings().inNamespace("workspace123").list();
     assertEquals(
         bindings
@@ -791,6 +901,7 @@ public class KubernetesNamespaceFactoryTest {
             "serviceAccount-metrics",
             "serviceAccount-cluster0",
             "serviceAccount-cluster1",
+            "serviceAccount-configmaps",
             "serviceAccount-view",
             "serviceAccount-exec",
             "serviceAccount-secrets"));
@@ -799,17 +910,18 @@ public class KubernetesNamespaceFactoryTest {
   @Test
   public void shouldCreateAndBindCredentialsSecretRole() throws Exception {
     // given
+    var serviceAccountConfigurator =
+        new WorkspaceServiceAccountConfigurator("serviceAccount", "cr2, cr3", clientFactory);
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "serviceAccount",
-                "cr2, cr3",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                Set.of(serviceAccountConfigurator),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -817,10 +929,10 @@ public class KubernetesNamespaceFactoryTest {
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
     prepareNamespace(toReturnNamespace);
-    when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
     when(clientFactory.create(any())).thenReturn(k8sClient);
+    when(cheClientFactory.create()).thenReturn(k8sClient);
 
     // when
     RuntimeIdentity identity =
@@ -861,14 +973,14 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "serviceAccount",
-                "",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                Set.of(
+                    new WorkspaceServiceAccountConfigurator("serviceAccount", "", clientFactory)),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -876,11 +988,11 @@ public class KubernetesNamespaceFactoryTest {
                 pool));
     KubernetesNamespace toReturnNamespace = mock(KubernetesNamespace.class);
     prepareNamespace(toReturnNamespace);
-    when(toReturnNamespace.getWorkspaceId()).thenReturn("workspace123");
     when(toReturnNamespace.getName()).thenReturn("workspace123");
     doReturn(toReturnNamespace).when(namespaceFactory).doCreateNamespaceAccess(any(), any());
     when(k8sClient.supportsApiPath(eq("/apis/metrics.k8s.io"))).thenReturn(true);
     when(clientFactory.create(any())).thenReturn(k8sClient);
+    when(cheClientFactory.create()).thenReturn(k8sClient);
 
     // when
     RuntimeIdentity identity =
@@ -888,7 +1000,6 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory.getOrCreate(identity);
 
     // then
-    verify(namespaceFactory).doCreateServiceAccount("workspace123", "workspace123");
 
     ServiceAccountList sas = k8sClient.serviceAccounts().inNamespace("workspace123").list();
     assertEquals(sas.getItems().size(), 1);
@@ -897,7 +1008,12 @@ public class KubernetesNamespaceFactoryTest {
     RoleList roles = k8sClient.rbac().roles().inNamespace("workspace123").list();
     assertEquals(
         roles.getItems().stream().map(r -> r.getMetadata().getName()).collect(Collectors.toSet()),
-        Sets.newHashSet("workspace-view", "workspace-metrics", "workspace-secrets", "exec"));
+        Sets.newHashSet(
+            "workspace-configmaps",
+            "workspace-view",
+            "workspace-metrics",
+            "workspace-secrets",
+            "exec"));
     Role role1 = roles.getItems().get(0);
     Role role2 = roles.getItems().get(1);
 
@@ -917,65 +1033,23 @@ public class KubernetesNamespaceFactoryTest {
             "serviceAccount-metrics",
             "serviceAccount-view",
             "serviceAccount-exec",
+            "serviceAccount-configmaps",
             "serviceAccount-secrets"));
   }
 
-  @Test
-  public void testNullClusterRolesResultsInEmptySet() {
-    namespaceFactory =
-        new KubernetesNamespaceFactory(
-            "",
-            null,
-            "che-<userid>",
-            true,
-            true,
-            true,
-            NAMESPACE_LABELS,
-            NAMESPACE_ANNOTATIONS,
-            clientFactory,
-            cheClientFactory,
-            userManager,
-            preferenceManager,
-            pool);
-    assertTrue(namespaceFactory.getClusterRoleNames().isEmpty());
-  }
-
-  @Test
-  public void testClusterRolesProperlyParsed() {
-    namespaceFactory =
-        new KubernetesNamespaceFactory(
-            "",
-            "  one,two, three ,,five  ",
-            "che-<userid>",
-            true,
-            true,
-            true,
-            NAMESPACE_LABELS,
-            NAMESPACE_ANNOTATIONS,
-            clientFactory,
-            cheClientFactory,
-            userManager,
-            preferenceManager,
-            pool);
-    Set<String> expected = Sets.newHashSet("one", "two", "three", "five");
-    assertTrue(namespaceFactory.getClusterRoleNames().containsAll(expected));
-    assertTrue(expected.containsAll(namespaceFactory.getClusterRoleNames()));
-  }
-
   @Test
   public void
       testEvalNamespaceUsesNamespaceDefaultIfWorkspaceDoesntRecordNamespaceAndLegacyNamespaceDoesntExist()
           throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -994,14 +1068,13 @@ public class KubernetesNamespaceFactoryTest {
   public void testEvalNamespaceUsesNamespaceFromUserPreferencesIfExist() throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1025,14 +1098,13 @@ public class KubernetesNamespaceFactoryTest {
       throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>-<username>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1057,14 +1129,13 @@ public class KubernetesNamespaceFactoryTest {
       throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>-<username>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1089,20 +1160,19 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "che-<username>",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
                 preferenceManager,
                 pool));
-    doReturn(Optional.empty()).when(namespaceFactory).fetchNamespace(anyString());
+    doReturn(empty()).when(namespaceFactory).fetchNamespace(anyString());
 
     String namespace =
         namespaceFactory.evaluateNamespaceName(
@@ -1117,14 +1187,13 @@ public class KubernetesNamespaceFactoryTest {
       throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1147,14 +1216,13 @@ public class KubernetesNamespaceFactoryTest {
   public void testEvalNamespaceTreatsWorkspaceRecordedNamespaceLiterally() throws Exception {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1198,14 +1266,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1225,14 +1292,13 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-che",
                 false,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -1265,14 +1331,13 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-cha-cha-cha",
                 false,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -1285,7 +1350,7 @@ public class KubernetesNamespaceFactoryTest {
     KubernetesNamespaceMetaImpl namespaceMeta =
         new KubernetesNamespaceMetaImpl(
             "jondoe-cha-cha-cha", ImmutableMap.of("phase", "active", "default", "true"));
-    doReturn(Optional.empty()).when(namespaceFactory).fetchNamespace(eq("jondoe-cha-cha-cha"));
+    doReturn(empty()).when(namespaceFactory).fetchNamespace(eq("jondoe-cha-cha-cha"));
 
     // when
     NamespaceResolutionContext context =
@@ -1304,14 +1369,13 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-cha-cha-cha",
                 false,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 NAMESPACE_ANNOTATIONS,
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -1355,14 +1419,13 @@ public class KubernetesNamespaceFactoryTest {
 
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             "try_placeholder_here=<username>",
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1381,14 +1444,13 @@ public class KubernetesNamespaceFactoryTest {
     namespaceFactory =
         spy(
             new KubernetesNamespaceFactory(
-                "",
-                "",
                 "<username>-che",
                 true,
                 true,
                 true,
                 NAMESPACE_LABELS,
                 "try_placeholder_here=<username>",
+                emptySet(),
                 clientFactory,
                 cheClientFactory,
                 userManager,
@@ -1413,14 +1475,13 @@ public class KubernetesNamespaceFactoryTest {
   public void normalizeTest(String raw, String expected) {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "<username>-che",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1433,14 +1494,13 @@ public class KubernetesNamespaceFactoryTest {
   public void normalizeLengthTest() {
     namespaceFactory =
         new KubernetesNamespaceFactory(
-            "",
-            "",
             "che-<userid>",
             true,
             true,
             true,
             NAMESPACE_LABELS,
             NAMESPACE_ANNOTATIONS,
+            emptySet(),
             clientFactory,
             cheClientFactory,
             userManager,
@@ -1520,12 +1580,15 @@ public class KubernetesNamespaceFactoryTest {
 
   private void prepareNamespace(KubernetesNamespace namespace) throws InfrastructureException {
     KubernetesSecrets secrets = mock(KubernetesSecrets.class);
-    when(namespace.secrets()).thenReturn(secrets);
+    lenient().when(namespace.secrets()).thenReturn(secrets);
+    KubernetesConfigsMaps configmaps = mock(KubernetesConfigsMaps.class);
+    lenient().when(namespace.secrets()).thenReturn(secrets);
+    lenient().when(namespace.configMaps()).thenReturn(configmaps);
     Secret secretMock = mock(Secret.class);
     ObjectMeta objectMeta = mock(ObjectMeta.class);
-    when(objectMeta.getName()).thenReturn(CREDENTIALS_SECRET_NAME);
-    when(secretMock.getMetadata()).thenReturn(objectMeta);
-    when(secrets.get()).thenReturn(Collections.singletonList(secretMock));
+    lenient().when(objectMeta.getName()).thenReturn(CREDENTIALS_SECRET_NAME);
+    lenient().when(secretMock.getMetadata()).thenReturn(objectMeta);
+    lenient().when(secrets.get()).thenReturn(Collections.singletonList(secretMock));
   }
 
   private Namespace createNamespace(String name, String phase) {
@@ -1541,6 +1604,6 @@ public class KubernetesNamespaceFactoryTest {
 
   private KubernetesNamespaceMeta testProvisioning(NamespaceResolutionContext context)
       throws InfrastructureException {
-    return new NamespaceProvisioner(namespaceFactory, emptySet()).provision(context);
+    return new NamespaceProvisioner(namespaceFactory).provision(context);
   }
 }
