@@ -16,20 +16,18 @@ import static java.lang.String.format;
 import static java.util.Collections.emptyList;
 import static java.util.Collections.emptyMap;
 import static org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta.PHASE_ATTRIBUTE;
-import static org.eclipse.che.workspace.infrastructure.kubernetes.namespace.AbstractWorkspaceServiceAccount.CREDENTIALS_SECRET_NAME;
 
 import com.google.common.annotations.VisibleForTesting;
 import com.google.inject.Inject;
 import com.google.inject.Singleton;
 import io.fabric8.kubernetes.api.model.ObjectMeta;
-import io.fabric8.kubernetes.api.model.Secret;
-import io.fabric8.kubernetes.api.model.SecretBuilder;
 import io.fabric8.kubernetes.client.KubernetesClientException;
 import io.fabric8.openshift.api.model.Project;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
+import java.util.Set;
 import java.util.stream.Collectors;
 import javax.inject.Named;
 import org.eclipse.che.api.core.model.workspace.Workspace;
@@ -44,11 +42,11 @@ import org.eclipse.che.workspace.infrastructure.kubernetes.CheServerKubernetesCl
 import org.eclipse.che.workspace.infrastructure.kubernetes.api.server.impls.KubernetesNamespaceMetaImpl;
 import org.eclipse.che.workspace.infrastructure.kubernetes.api.shared.KubernetesNamespaceMeta;
 import org.eclipse.che.workspace.infrastructure.kubernetes.namespace.KubernetesNamespaceFactory;
+import org.eclipse.che.workspace.infrastructure.kubernetes.namespace.configurator.NamespaceConfigurator;
 import org.eclipse.che.workspace.infrastructure.kubernetes.util.KubernetesSharedPool;
 import org.eclipse.che.workspace.infrastructure.openshift.CheServerOpenshiftClientFactory;
 import org.eclipse.che.workspace.infrastructure.openshift.Constants;
 import org.eclipse.che.workspace.infrastructure.openshift.OpenShiftClientFactory;
-import org.eclipse.che.workspace.infrastructure.openshift.provision.OpenShiftStopWorkspaceRoleProvisioner;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -64,14 +62,11 @@ public class OpenShiftProjectFactory extends KubernetesNamespaceFactory {
   private final boolean initWithCheServerSa;
   private final OpenShiftClientFactory clientFactory;
   private final CheServerOpenshiftClientFactory cheOpenShiftClientFactory;
-  private final OpenShiftStopWorkspaceRoleProvisioner stopWorkspaceRoleProvisioner;
 
   private final String oAuthIdentityProvider;
 
   @Inject
   public OpenShiftProjectFactory(
-      @Nullable @Named("che.infra.kubernetes.service_account_name") String serviceAccountName,
-      @Nullable @Named("che.infra.kubernetes.workspace_sa_cluster_roles") String clusterRoleNames,
       @Nullable @Named("che.infra.kubernetes.namespace.default") String defaultNamespaceName,
       @Named("che.infra.kubernetes.namespace.creation_allowed") boolean namespaceCreationAllowed,
       @Named("che.infra.kubernetes.namespace.label") boolean labelProjects,
@@ -79,24 +74,23 @@ public class OpenShiftProjectFactory extends KubernetesNamespaceFactory {
       @Named("che.infra.kubernetes.namespace.labels") String projectLabels,
       @Named("che.infra.kubernetes.namespace.annotations") String projectAnnotations,
       @Named("che.infra.openshift.project.init_with_server_sa") boolean initWithCheServerSa,
+      Set<NamespaceConfigurator> namespaceConfigurators,
       OpenShiftClientFactory clientFactory,
       CheServerKubernetesClientFactory cheClientFactory,
       CheServerOpenshiftClientFactory cheOpenShiftClientFactory,
-      OpenShiftStopWorkspaceRoleProvisioner stopWorkspaceRoleProvisioner,
       UserManager userManager,
       PreferenceManager preferenceManager,
       KubernetesSharedPool sharedPool,
       @Nullable @Named("che.infra.openshift.oauth_identity_provider")
           String oAuthIdentityProvider) {
     super(
-        serviceAccountName,
-        clusterRoleNames,
         defaultNamespaceName,
         namespaceCreationAllowed,
         labelProjects,
         annotateProjects,
         projectLabels,
         projectAnnotations,
+        namespaceConfigurators,
         clientFactory,
         cheClientFactory,
         userManager,
@@ -105,15 +99,16 @@ public class OpenShiftProjectFactory extends KubernetesNamespaceFactory {
     this.initWithCheServerSa = initWithCheServerSa;
     this.clientFactory = clientFactory;
     this.cheOpenShiftClientFactory = cheOpenShiftClientFactory;
-    this.stopWorkspaceRoleProvisioner = stopWorkspaceRoleProvisioner;
     this.oAuthIdentityProvider = oAuthIdentityProvider;
   }
 
   public OpenShiftProject getOrCreate(RuntimeIdentity identity) throws InfrastructureException {
     OpenShiftProject osProject = get(identity);
 
+    var subject = EnvironmentContext.getCurrent().getSubject();
     NamespaceResolutionContext resolutionCtx =
-        new NamespaceResolutionContext(EnvironmentContext.getCurrent().getSubject());
+        new NamespaceResolutionContext(
+            identity.getWorkspaceId(), subject.getUserId(), subject.getUserName());
     Map<String, String> namespaceAnnotationsEvaluated =
         evaluateAnnotationPlaceholders(resolutionCtx);
 
@@ -123,35 +118,8 @@ public class OpenShiftProjectFactory extends KubernetesNamespaceFactory {
         labelNamespaces ? namespaceLabels : emptyMap(),
         annotateNamespaces ? namespaceAnnotationsEvaluated : emptyMap());
 
-    // create credentials secret
-    if (osProject
-        .secrets()
-        .get()
-        .stream()
-        .noneMatch(s -> s.getMetadata().getName().equals(CREDENTIALS_SECRET_NAME))) {
-      Secret secret =
-          new SecretBuilder()
-              .withType("opaque")
-              .withNewMetadata()
-              .withName(CREDENTIALS_SECRET_NAME)
-              .endMetadata()
-              .build();
-      clientFactory
-          .createOC()
-          .secrets()
-          .inNamespace(identity.getInfrastructureNamespace())
-          .create(secret);
-    }
+    configureNamespace(resolutionCtx, osProject.getName());
 
-    if (!isNullOrEmpty(getServiceAccountName())) {
-      OpenShiftWorkspaceServiceAccount osWorkspaceServiceAccount =
-          doCreateServiceAccount(osProject.getWorkspaceId(), osProject.getName());
-      osWorkspaceServiceAccount.prepare();
-    }
-
-    if (!isNullOrEmpty(oAuthIdentityProvider)) {
-      stopWorkspaceRoleProvisioner.provision(osProject.getName());
-    }
     return osProject;
   }
 
@@ -172,11 +140,6 @@ public class OpenShiftProjectFactory extends KubernetesNamespaceFactory {
     }
   }
 
-  @Override
-  protected boolean checkNamespaceExists(String namespaceName) throws InfrastructureException {
-    return fetchNamespaceObject(namespaceName).isPresent();
-  }
-
   /**
    * Creates a kubernetes namespace for the specified workspace.
    *
@@ -200,12 +163,6 @@ public class OpenShiftProjectFactory extends KubernetesNamespaceFactory {
         workspaceId);
   }
 
-  @VisibleForTesting
-  OpenShiftWorkspaceServiceAccount doCreateServiceAccount(String workspaceId, String projectName) {
-    return new OpenShiftWorkspaceServiceAccount(
-        workspaceId, projectName, getServiceAccountName(), getClusterRoleNames(), clientFactory);
-  }
-
   @Override
   public Optional<KubernetesNamespaceMeta> fetchNamespace(String name)
       throws InfrastructureException {
